@@ -46,7 +46,6 @@ void test_circuitpadding_conditions(void *arg);
 void test_circuitpadding_serialize(void *arg);
 void test_circuitpadding_rtt(void *arg);
 void test_circuitpadding_tokens(void *arg);
-void test_circuitpadding_circuitsetup_machine(void *arg);
 
 static void
 simulate_single_hop_extend(circuit_t *client, circuit_t *mid_relay,
@@ -321,12 +320,12 @@ test_circuitpadding_rtt(void *arg)
 
   /* Test 1: Test measuring RTT */
   circpad_cell_event_nonpadding_received((circuit_t*)relay_side);
-  tt_int_op(relay_side->padding_info[0]->last_received_time_usec, OP_NE, 0);
+  tt_u64_op(relay_side->padding_info[0]->last_received_time_usec, OP_NE, 0);
 
   timers_advance_and_run(20);
 
   circpad_cell_event_nonpadding_sent((circuit_t*)relay_side);
-  tt_int_op(relay_side->padding_info[0]->last_received_time_usec, OP_EQ, 0);
+  tt_u64_op(relay_side->padding_info[0]->last_received_time_usec, OP_EQ, 0);
 
   tt_int_op(relay_side->padding_info[0]->rtt_estimate_usec, OP_GE, 19000);
   tt_int_op(relay_side->padding_info[0]->rtt_estimate_usec, OP_LE, 30000);
@@ -338,11 +337,11 @@ test_circuitpadding_rtt(void *arg)
 
   circpad_cell_event_nonpadding_received((circuit_t*)relay_side);
   circpad_cell_event_nonpadding_received((circuit_t*)relay_side);
-  tt_int_op(relay_side->padding_info[0]->last_received_time_usec, OP_NE, 0);
+  tt_u64_op(relay_side->padding_info[0]->last_received_time_usec, OP_NE, 0);
   timers_advance_and_run(20);
   circpad_cell_event_nonpadding_sent((circuit_t*)relay_side);
   circpad_cell_event_nonpadding_sent((circuit_t*)relay_side);
-  tt_int_op(relay_side->padding_info[0]->last_received_time_usec, OP_EQ, 0);
+  tt_u64_op(relay_side->padding_info[0]->last_received_time_usec, OP_EQ, 0);
 
   tt_int_op(relay_side->padding_info[0]->rtt_estimate_usec, OP_GE, 20000);
   tt_int_op(relay_side->padding_info[0]->rtt_estimate_usec, OP_LE, 21000);
@@ -362,7 +361,7 @@ test_circuitpadding_rtt(void *arg)
 
   tt_int_op(relay_side->padding_info[0]->rtt_estimate_usec, OP_EQ,
             rtt_estimate);
-  tt_int_op(relay_side->padding_info[0]->last_received_time_usec, OP_EQ, 0);
+  tt_u64_op(relay_side->padding_info[0]->last_received_time_usec, OP_EQ, 0);
   tt_int_op(relay_side->padding_info[0]->stop_rtt_update, OP_EQ, 1);
   tt_int_op(circpad_histogram_bin_to_usec(relay_side->padding_info[0], 0),
             OP_EQ,
@@ -372,11 +371,11 @@ test_circuitpadding_rtt(void *arg)
 
   /* Test 3: Make sure client side machine properly ignores RTT */
   circpad_cell_event_nonpadding_received((circuit_t*)client_side);
-  tt_int_op(client_side->padding_info[0]->last_received_time_usec, OP_EQ, 0);
+  tt_u64_op(client_side->padding_info[0]->last_received_time_usec, OP_EQ, 0);
 
   timers_advance_and_run(20);
   circpad_cell_event_nonpadding_sent((circuit_t*)client_side);
-  tt_int_op(client_side->padding_info[0]->last_received_time_usec, OP_EQ, 0);
+  tt_u64_op(client_side->padding_info[0]->last_received_time_usec, OP_EQ, 0);
 
   tt_int_op(client_side->padding_info[0]->rtt_estimate_usec, OP_EQ, 0);
   tt_int_op(circpad_histogram_bin_to_usec(client_side->padding_info[0], 0),
@@ -1239,6 +1238,10 @@ test_circuitpadding_wronghop(void *arg)
 
   MOCK(circuitmux_attach_circuit, circuitmux_attach_circuit_mock);
 
+  /* Mock this function so that our cell counting tests don't get confused by
+   * padding that gets sent by scheduled timers. */
+  MOCK(circpad_machine_schedule_padding,circpad_machine_schedule_padding_mock);
+
   client_side = (circuit_t *)origin_circuit_new();
   dummy_channel.cmux = circuitmux_alloc();
   relay_side = (circuit_t *)new_fake_orcirc(&dummy_channel,
@@ -1805,6 +1808,8 @@ test_circuitpadding_conditions(void *arg)
   return;
 }
 
+/** Disabled unstable test until #29298 is implemented (see #29122) */
+#if 0
 void
 test_circuitpadding_circuitsetup_machine(void *arg)
 {
@@ -1857,7 +1862,7 @@ test_circuitpadding_circuitsetup_machine(void *arg)
   tt_int_op(relay_side->padding_info[0]->current_state, OP_EQ,
           CIRCPAD_STATE_BURST);
 
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
   tt_int_op(relay_side->padding_info[0]->is_padding_timer_scheduled,
             OP_EQ, 0);
@@ -1868,73 +1873,73 @@ test_circuitpadding_circuitsetup_machine(void *arg)
   tt_int_op(relay_side->padding_info[0]->current_state, OP_EQ,
               CIRCPAD_STATE_GAP);
 
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_EQ, 0);
-  tt_int_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
   timers_advance_and_run(5000);
   tt_int_op(n_client_cells, OP_EQ, 2);
   tt_int_op(n_relay_cells, OP_EQ, 2);
 
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
-  tt_int_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
             OP_EQ, 0);
   timers_advance_and_run(2000);
   tt_int_op(n_client_cells, OP_EQ, 3);
   tt_int_op(n_relay_cells, OP_EQ, 2);
 
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_EQ, 0);
-  tt_int_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
   timers_advance_and_run(5000);
   tt_int_op(n_client_cells, OP_EQ, 3);
   tt_int_op(n_relay_cells, OP_EQ, 3);
 
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
-  tt_int_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
             OP_EQ, 0);
   timers_advance_and_run(2000);
   tt_int_op(n_client_cells, OP_EQ, 4);
   tt_int_op(n_relay_cells, OP_EQ, 3);
 
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_EQ, 0);
-  tt_int_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
   timers_advance_and_run(5000);
   tt_int_op(n_client_cells, OP_EQ, 4);
   tt_int_op(n_relay_cells, OP_EQ, 4);
 
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
-  tt_int_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
             OP_EQ, 0);
   timers_advance_and_run(2000);
   tt_int_op(n_client_cells, OP_EQ, 5);
   tt_int_op(n_relay_cells, OP_EQ, 4);
 
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_EQ, 0);
-  tt_int_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
   timers_advance_and_run(5000);
   tt_int_op(n_client_cells, OP_EQ, 5);
   tt_int_op(n_relay_cells, OP_EQ, 5);
 
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
-  tt_int_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
             OP_EQ, 0);
   timers_advance_and_run(2000);
   tt_int_op(n_client_cells, OP_EQ, 6);
   tt_int_op(n_relay_cells, OP_EQ, 5);
 
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_EQ, 0);
-  tt_int_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
   timers_advance_and_run(5000);
   tt_int_op(n_client_cells, OP_EQ, 6);
@@ -1942,11 +1947,11 @@ test_circuitpadding_circuitsetup_machine(void *arg)
 
   tt_int_op(client_side->padding_info[0]->current_state,
             OP_EQ, CIRCPAD_STATE_END);
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_EQ, 0);
   tt_int_op(relay_side->padding_info[0]->current_state,
             OP_EQ, CIRCPAD_STATE_GAP);
-  tt_int_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
             OP_EQ, 0);
 
   /* Verify we can't schedule padding in END state */
@@ -1982,17 +1987,17 @@ test_circuitpadding_circuitsetup_machine(void *arg)
 
   tt_int_op(n_client_cells, OP_EQ, 8);
   tt_int_op(n_relay_cells, OP_EQ, 8);
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
-  tt_int_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
 
   /* Test timer cancel due to state rules */
   circpad_cell_event_nonpadding_sent(client_side);
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_EQ, 0);
   circpad_cell_event_padding_received(client_side);
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
 
   /* Simulate application traffic to cancel timer */
@@ -2026,9 +2031,9 @@ test_circuitpadding_circuitsetup_machine(void *arg)
   tt_int_op(relay_side->padding_info[0]->current_state, OP_EQ,
           CIRCPAD_STATE_GAP);
 
-  tt_int_op(client_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(client_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
-  tt_int_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
+  tt_u64_op(relay_side->padding_info[0]->padding_scheduled_at_usec,
             OP_NE, 0);
   circuit_mark_for_close(client_side, END_CIRC_REASON_FLAG_REMOTE);
   free_fake_orcirc(relay_side);
@@ -2050,6 +2055,7 @@ test_circuitpadding_circuitsetup_machine(void *arg)
 
   return;
 }
+#endif
 
 /** Helper function: Initializes a padding machine where every state uses the
  *  uniform probability distribution.  */
@@ -2336,11 +2342,11 @@ test_circuitpadding_global_rate_limiting(void *arg)
     { #name, test_##name, (flags), NULL, NULL }
 
 struct testcase_t circuitpadding_tests[] = {
-  //TEST_CIRCUITPADDING(circuitpadding_circuitsetup_machine, 0),
   TEST_CIRCUITPADDING(circuitpadding_tokens, TT_FORK),
   TEST_CIRCUITPADDING(circuitpadding_negotiation, TT_FORK),
   TEST_CIRCUITPADDING(circuitpadding_wronghop, TT_FORK),
-  TEST_CIRCUITPADDING(circuitpadding_circuitsetup_machine, TT_FORK),
+  /** Disabled unstable test until #29298 is implemented (see #29122) */
+  //  TEST_CIRCUITPADDING(circuitpadding_circuitsetup_machine, TT_FORK),
   TEST_CIRCUITPADDING(circuitpadding_conditions, TT_FORK),
   TEST_CIRCUITPADDING(circuitpadding_rtt, TT_FORK),
   TEST_CIRCUITPADDING(circuitpadding_sample_distribution, TT_FORK),
@@ -2353,4 +2359,3 @@ struct testcase_t circuitpadding_tests[] = {
   TEST_CIRCUITPADDING(circuitpadding_token_removal_exact, TT_FORK),
   END_OF_TESTCASES
 };
-
